from pypdf import PdfReader, PdfWriter
from pypdf._page import Transformation


def resize_pdf_page(page, scale_factor):
    """
    Resize or rescale a single PDF page, including its content and canvas.

    Args:
        page (PageObject): The page to be resized.
        scale_factor (float): The factor by which to scale the page.
                              e.g., 0.5 for 50% size, 2.0 for 200% size.

    Returns:
        PageObject: The resized page.
    """
    # Apply scaling transformation to the page content
    transformation = Transformation().scale(scale_factor, scale_factor)
    page.add_transformation(transformation)

    # Adjust the media box to reflect the new dimensions
    media_box = page.mediabox
    new_width = media_box.width * scale_factor
    new_height = media_box.height * scale_factor
    page.mediabox.lower_left = (media_box.lower_left[0], media_box.lower_left[1])
    page.mediabox.upper_right = (new_width, new_height)

    return page


def split_pdf_vertically(
    input_pdf,
    output_pdf_prefix,
    cut_points=None,
    new_width: float = 0,
    single_output=True,
):
    """
    Splits a PDF vertically and optionally resizes it.

    Args:
        input_pdf (str): Path to the input PDF file.
        output_pdf_prefix (str): Prefix for output PDF files.
        cut_points (list of tuples): List of (start_y, end_y) ratios for vertical splitting.
        new_width (float): Desired width of the output PDF in inches. Default is no resizing.
        single_output (bool): If True, outputs a single PDF with multiple pages.
                              If False, outputs a separate PDF for each segment.
    """
    if cut_points is None:
        cut_points = [(0, 1)]

    reader = PdfReader(input_pdf)
    scale_factor = 1.0
    original_page = reader.pages[0]

    # Resize the page if a new width is specified
    if new_width:
        scale_factor = (new_width * 72) / original_page.mediabox.width
        original_page = resize_pdf_page(original_page, scale_factor)

    media_box = original_page.mediabox
    h = original_page.mediabox.height

    # Create a single writer for combined output, if needed
    combined_writer = PdfWriter() if single_output else None
    if not new_width:
        _nw = original_page.mediabox.width // 72
    else:
        _nw = new_width
    for i, (start_y, end_y) in enumerate(reversed(cut_points)):
        # Convert relative coordinates to absolute coordinates
        start_y = int(h * start_y)
        end_y = int(h * end_y)

        # Clone and crop the page
        writer = PdfWriter()
        new_page = original_page.clone(writer)
        new_page.mediabox.upper_right = (media_box.upper_right[0], start_y)
        new_page.mediabox.lower_left = (media_box.lower_left[0], end_y)
        if single_output:
            combined_writer.add_page(new_page)
        else:
            writer.add_page(new_page)
            with open(
                f"{output_pdf_prefix}W{_nw:03d}_{start_y}-{end_y}.pdf", "wb"
            ) as f:
                writer.write(f)

    # Write the single output file, if applicable
    if single_output:
        _ct_pts = set(sum([[*k] for k in cut_points], []))
        ct = "-".join(sorted(list(map(lambda c: str(int(c * h)), _ct_pts))))

        with open(f"{output_pdf_prefix}W{_nw:03d}_{ct}_combined.pdf", "wb") as f:
            combined_writer.write(f)


if __name__ == "__main__":
    # Usage:
    input_pdf = "sample.pdf"
    output_pdf_prefix = "split"
    # Specify (start_y, end_y) for each segment
    cut_points = [(0, 0.0949358), (0.0949358, 0.2)]
    split_pdf_vertically(
        input_pdf, output_pdf_prefix, cut_points, new_width=None, single_output=False
    )
    split_pdf_vertically(
        input_pdf, "resize", cut_points=None, new_width=32, single_output=True
    )
    # split_pdf_vertically(
    #     input_pdf,
    #     output_pdf_prefix,
    #     cut_points=[(0, 0.25), (0.25, 0.5), (0.5, 0.75), (0.75, 1)],
    #     new_width=32,
    #     single_output=False,
    # )
